@@ -350,7 +350,7 @@ def get_config_from_root(root):
         return None
     cfg = VersioneerConfig()
     cfg.VCS = VCS
-    cfg.style = get(parser, "style") or ""
+    cfg.style = get(parser, "style") or "pep440-auto"
     cfg.versionfile_source = get(parser, "versionfile_source")
     cfg.versionfile_build = get(parser, "versionfile_build")
     cfg.tag_prefix = get(parser, "tag_prefix")
@@ -418,6 +418,18 @@ def run_command(commands, args, cwd=None, verbose=False, hide_stderr=False,
     return stdout, p.returncode
 
 
+def unique(seq):
+    """Find unique elements in a list, without sorting
+
+    Implementation does not depend on `numpy`
+    * borrowed from https://stackoverflow.com/a/480227
+    * credit to Markus Jarderot
+    """
+    seen = set()
+    seen_add = seen.add
+    return [x for x in seq if not (x in seen or seen_add(x))]
+
+
 LONG_VERSION_PY['git'] = '''
 # This file helps to compute a version number in source trees obtained from
 # git-archive tarball (such as those provided by githubs download-from-tag
@@ -460,6 +472,8 @@ def get_config():
     # _version.py
     cfg = VersioneerConfig()
     cfg.VCS = "git"
+    # cfg.style = "pep440-micro"
+    # cfg.style = "pep440-develop"
     cfg.style = "pep440-auto"
     cfg.tag_prefix = ""
     cfg.parentdir_prefix = ""
@@ -684,6 +698,11 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
 
     # now we have TAG-NUM-gHEX or HEX
 
+    #  Determine which branch is active (ie, checked-out)
+    branch_name, rc = run_command(
+        GITS, ["rev-parse", "--abbrev-ref", "HEAD"], cwd=root)
+    pieces["branch"] = branch_name
+
     if "-" in git_describe:
         # TAG-NUM-gHEX
         mo = re.search(r'^(.+)-(\d+)-g([0-9a-f]+)$', git_describe)
@@ -708,9 +727,13 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
         pieces["distance"] = int(mo.group(2))
 
         # Added to deal with merged branches
+        history_search_args = ["--ancestry-path"]
+        if (branch_name == "master"):
+            # Count distance only along `master`; ignore merge paths
+            history_search_args = ["--ancestry-path", "--first-parent"]
         distance_to_tag, rc = run_command(
-            GITS, ["rev-list", "--count", "--ancestry-path",
-                   "%s..HEAD" % pieces["closest-tag"] ], cwd=root)
+            GITS, ["rev-list", "--count",] + history_search_args +
+                  ["%s..HEAD" % pieces["closest-tag"] ], cwd=root)
         pieces["distance"] = int(distance_to_tag)
 
         # commit: short hex revision ID
@@ -725,9 +748,6 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
 
     ##################################################
     #  Added for `develop` branch from `master`
-    branch_name, rc = run_command(
-        GITS, ["rev-parse", "--abbrev-ref", "HEAD"], cwd=root)
-    pieces["branch"] = branch_name
     # `HEAD` might have branched from untagged commit on `master`
     merge_base_master, rc = run_command(
         GITS, ["merge-base", "master", "HEAD", "-a"], cwd=root)
@@ -736,6 +756,18 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
         "%s..HEAD" % merge_base_master], cwd=root)
     pieces["distance-to-master"] = int(distance_to_master)
 
+    ##################################################
+    #  Added for branches other than `master`
+    if pieces["closest-tag"]:
+        # TAG exists
+        master_to_tag, rc = run_command(
+            GITS, ["rev-list", "--count", "--ancestry-path", "--first-parent",
+                   "%s..%s" % (pieces["closest-tag"], merge_base_master) ],
+            cwd=root)
+        pieces["master-to-tag"] = int(master_to_tag)
+    else:
+        pieces["master-to-tag"] = pieces["distance"]
+
     ##################################################
     # Added for "feature" branch from `develop`
     pieces["develop"] = None
@@ -895,7 +927,8 @@ def render_pep440_develop(pieces):
     if pieces["closest-tag"]:
         rendered = pieces["closest-tag"]
         distance_to_merge = pieces["distance-to-master"]
-        distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        # distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        distance_merge_to_tag = pieces["master-to-tag"]
         if (distance_merge_to_tag > 0):
             rendered += ".%d" % distance_merge_to_tag
         rendered += ".post.dev%d" % distance_to_merge
@@ -925,7 +958,8 @@ def render_pep440_feature(pieces):
         rendered = pieces["closest-tag"]
         distance_to_develop = pieces["distance-to-develop"]
         distance_to_merge = pieces["distance-to-master"]
-        distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        # distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        distance_merge_to_tag = pieces["master-to-tag"]
         distance_dev_to_merge = (distance_to_merge - distance_to_develop)
         if (distance_merge_to_tag > 0):
             rendered += ".%d" % distance_merge_to_tag
@@ -1228,6 +1262,11 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
 
     # now we have TAG-NUM-gHEX or HEX
 
+    #  Determine which branch is active (ie, checked-out)
+    branch_name, rc = run_command(
+        GITS, ["rev-parse", "--abbrev-ref", "HEAD"], cwd=root)
+    pieces["branch"] = branch_name
+
     if "-" in git_describe:
         # TAG-NUM-gHEX
         mo = re.search(r'^(.+)-(\d+)-g([0-9a-f]+)$', git_describe)
@@ -1252,9 +1291,13 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
         pieces["distance"] = int(mo.group(2))
 
         # Added to deal with merged branches
+        history_search_args = ["--ancestry-path"]
+        if (branch_name == "master"):
+            # Count distance only along `master`; ignore merge paths
+            history_search_args = ["--ancestry-path", "--first-parent"]
         distance_to_tag, rc = run_command(
-            GITS, ["rev-list", "--count", "--ancestry-path",
-                   "%s..HEAD" % pieces["closest-tag"] ], cwd=root)
+            GITS, ["rev-list", "--count",] + history_search_args +
+                  ["%s..HEAD" % pieces["closest-tag"] ], cwd=root)
         pieces["distance"] = int(distance_to_tag)
 
         # commit: short hex revision ID
@@ -1269,9 +1312,6 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
 
     ##################################################
     #  Added for `develop` branch from `master`
-    branch_name, rc = run_command(
-        GITS, ["rev-parse", "--abbrev-ref", "HEAD"], cwd=root)
-    pieces["branch"] = branch_name
     # `HEAD` might have branched from untagged commit on `master`
     merge_base_master, rc = run_command(
         GITS, ["merge-base", "master", "HEAD", "-a"], cwd=root)
@@ -1280,6 +1320,18 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
         "%s..HEAD" % merge_base_master], cwd=root)
     pieces["distance-to-master"] = int(distance_to_master)
 
+    ##################################################
+    #  Added for branches other than `master`
+    if pieces["closest-tag"]:
+        # TAG exists
+        master_to_tag, rc = run_command(
+            GITS, ["rev-list", "--count", "--ancestry-path", "--first-parent",
+                   "%s..%s" % (pieces["closest-tag"], merge_base_master) ],
+            cwd=root)
+        pieces["master-to-tag"] = int(master_to_tag)
+    else:
+        pieces["master-to-tag"] = pieces["distance"]
+
     ##################################################
     # Added for "feature" branch from `develop`
     pieces["develop"] = None
@@ -1301,6 +1353,13 @@ def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
                        cwd=root)[0].strip()
     pieces["date"] = date.strip().replace(" ", "T", 1).replace(" ", "", 1)
 
+    ##################################################
+    # Added to retrieve list of authors
+    (authors_raw, rc) = run_command(GITS, ["log", "--pretty=%an"], cwd=root)
+    authors = [author.strip() for author in authors_raw.split('\n')]
+    authors_unique = unique(authors)
+    pieces["authors"] = list(reversed(authors_unique))
+
     return pieces
 
 
@@ -1548,7 +1607,8 @@ def render_pep440_develop(pieces):
     if pieces["closest-tag"]:
         rendered = pieces["closest-tag"]
         distance_to_merge = pieces["distance-to-master"]
-        distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        # distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        distance_merge_to_tag = pieces["master-to-tag"]
         if (distance_merge_to_tag > 0):
             rendered += ".%d" % distance_merge_to_tag
         rendered += ".post.dev%d" % distance_to_merge
@@ -1578,7 +1638,8 @@ def render_pep440_feature(pieces):
         rendered = pieces["closest-tag"]
         distance_to_develop = pieces["distance-to-develop"]
         distance_to_merge = pieces["distance-to-master"]
-        distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        # distance_merge_to_tag = (pieces["distance"] - distance_to_merge)
+        distance_merge_to_tag = pieces["master-to-tag"]
         distance_dev_to_merge = (distance_to_merge - distance_to_develop)
         if (distance_merge_to_tag > 0):
             rendered += ".%d" % distance_merge_to_tag
@@ -1668,10 +1729,12 @@ def render(pieces, style):
                 "full-revisionid": pieces.get("long"),
                 "dirty": None,
                 "error": pieces["error"],
-                "date": None}
+                "date": None,
+                "authors": None,
+                }
 
     if not style or style == "default":
-        style = "pep440"  # the default
+        style = "pep440-auto"  # the default
 
     if style == "pep440":
         rendered = render_pep440(pieces)
@@ -1696,7 +1759,7 @@ def render(pieces, style):
 
     return {"version": rendered, "full-revisionid": pieces["long"],
             "dirty": pieces["dirty"], "error": None,
-            "date": pieces.get("date")}
+            "date": pieces.get("date"), "authors": pieces["authors"]}
 
 
 class VersioneerBadRootError(Exception):
@@ -1776,7 +1839,9 @@ def get_versions(verbose=False):
 
     return {"version": "0+unknown", "full-revisionid": None,
             "dirty": None, "error": "unable to compute version",
-            "date": None}
+            "date": None,
+            "authors": [],
+            }
 
 
 def get_version():
